@@ -91,6 +91,8 @@ nemo_find_file_insensitive (GFile *parent, const gchar *name);
 
 char * nemo_get_accel_map_file (void);
 
+char * nemo_get_scripts_directory_path (void);
+
 GHashTable * nemo_trashed_files_get_original_directories (GList *files,
 							      GList **unhandled_files);
 void nemo_restore_files_from_trash (GList *files,
@@ -104,7 +106,4 @@ void nemo_get_x_content_types_for_mount_async (GMount *mount,
 						   GCancellable *cancellable,
 						   gpointer user_data);
 
-GIcon * nemo_special_directory_get_icon (GUserDirectory directory);
-GIcon * nemo_special_directory_get_symbolic_icon (GUserDirectory directory);
-
 #endif /* NEMO_FILE_UTILITIES_H */
