@@ -176,6 +176,8 @@ gboolean                nemo_file_contains_text                     (NemoFile
 char *                  nemo_file_get_display_name                  (NemoFile                   *file);
 char *                  nemo_file_get_edit_name                     (NemoFile                   *file);
 char *                  nemo_file_get_name                          (NemoFile                   *file);
+const char *            nemo_file_peek_name                         (NemoFile                   *file);
+
 GFile *                 nemo_file_get_location                      (NemoFile                   *file);
 char *			 nemo_file_get_description			 (NemoFile			 *file);
 char *                  nemo_file_get_uri                           (NemoFile                   *file);
@@ -231,10 +233,6 @@ gboolean                nemo_file_should_show_type                  (NemoFile
 GList *                 nemo_file_get_keywords                      (NemoFile                   *file);
 GList *                 nemo_file_get_emblem_icons                  (NemoFile                   *file,
 									 char                          **exclude);
-char *                  nemo_file_get_top_left_text                 (NemoFile                   *file);
-char *                  nemo_file_peek_top_left_text                (NemoFile                   *file,
-									 gboolean                        need_large_text,
-									 gboolean                       *got_top_left_text);
 gboolean                nemo_file_get_directory_item_mime_types     (NemoFile                   *file,
 									 GList                         **mime_list);
 
@@ -454,7 +452,10 @@ GFile *                 nemo_file_get_activation_location           (NemoFile
 char *                  nemo_file_get_drop_target_uri               (NemoFile                   *file);
 
 GIcon *                 nemo_file_get_gicon                         (NemoFile                   *file,
-									 NemoFileIconFlags           flags);
+                                                                     NemoFileIconFlags           flags);
+GIcon *                 nemo_file_get_emblemed_icon                 (NemoFile                   *file,
+                                                                     NemoFileIconFlags           flags);
+
 NemoIconInfo *      nemo_file_get_icon                          (NemoFile                   *file,
 									 int                             size,
                                      int                             scale,
@@ -497,6 +498,13 @@ char *   nemo_file_get_date_as_string             (NemoFile *file, NemoDateType
 
 gchar *  nemo_file_construct_tooltip              (NemoFile *file, NemoFileTooltipFlags flags);
 
+gboolean nemo_file_has_thumbnail_access_problem   (NemoFile *file);
+
+gint     nemo_file_get_monitor_number             (NemoFile *file);
+
+gboolean nemo_file_get_is_desktop_orphan          (NemoFile *file);
+void     nemo_file_set_is_desktop_orphan          (NemoFile *file, gboolean is_desktop_orphan);
+
 /* Debugging */
 void                    nemo_file_dump                              (NemoFile                   *file);
 
