@@ -44,7 +44,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <signal.h>
-#include <libgnome-desktop/gnome-desktop-thumbnail.h>
+#include <libcinnamon-desktop/gnome-desktop-thumbnail.h>
 
 #include "nemo-file-private.h"
 
@@ -59,6 +59,11 @@
 /* Cool-off period between last file modification time and thumbnail creation */
 #define THUMBNAIL_CREATION_DELAY_SECS 3
 
+#define NEMO_THUMBNAIL_FRAME_LEFT 3
+#define NEMO_THUMBNAIL_FRAME_TOP 3
+#define NEMO_THUMBNAIL_FRAME_RIGHT 3
+#define NEMO_THUMBNAIL_FRAME_BOTTOM 3
+
 static gpointer thumbnail_thread_start (gpointer data);
 
 /* structure used for making thumbnails, associating a uri with where the thumbnail is to be stored */
@@ -67,6 +72,7 @@ typedef struct {
 	char *image_uri;
 	char *mime_type;
 	time_t original_file_mtime;
+    gint throttle_count;
 } NemoThumbnailInfo;
 
 /*
@@ -165,9 +171,7 @@ thumbnail_thread_starter_cb (gpointer data)
 	pthread_attr_init (&thread_attributes);
 	pthread_attr_setdetachstate (&thread_attributes,
 				     PTHREAD_CREATE_DETACHED);
-#ifdef _POSIX_THREAD_ATTR_STACKSIZE
-	pthread_attr_setstacksize (&thread_attributes, 128*1024);
-#endif
+
 #ifdef DEBUG_THUMBNAILS
 	g_message ("(Main Thread) Creating thumbnails thread\n");
 #endif
@@ -185,58 +189,6 @@ thumbnail_thread_starter_cb (gpointer data)
 	return FALSE;
 }
 
-void
-nemo_update_thumbnail_file_copied (const char *source_file_uri,
-				       const char *destination_file_uri)
-{
-	char *old_thumbnail_path;
-	time_t mtime;
-	GdkPixbuf *pixbuf;
-	GnomeDesktopThumbnailFactory *factory;
-	
-	old_thumbnail_path = gnome_desktop_thumbnail_path_for_uri (source_file_uri, GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
-	if (old_thumbnail_path != NULL &&
-	    g_file_test (old_thumbnail_path, G_FILE_TEST_EXISTS)) {
-		if (get_file_mtime (destination_file_uri, &mtime)) {
-			pixbuf = gdk_pixbuf_new_from_file (old_thumbnail_path, NULL);
-			
-			if (pixbuf && gnome_desktop_thumbnail_has_uri (pixbuf, source_file_uri)) {
-				factory = get_thumbnail_factory ();
-				gnome_desktop_thumbnail_factory_save_thumbnail (factory,
-										pixbuf,
-										destination_file_uri,
-										mtime);
-			}
-			
-			if (pixbuf) {
-				g_object_unref (pixbuf);
-			}
-		}
-	}
-
-	g_free (old_thumbnail_path);
-}
-
-void
-nemo_update_thumbnail_file_renamed (const char *source_file_uri,
-					const char *destination_file_uri)
-{
-	nemo_update_thumbnail_file_copied (source_file_uri, destination_file_uri);
-	nemo_remove_thumbnail_for_file (source_file_uri);
-}
-
-void 
-nemo_remove_thumbnail_for_file (const char *file_uri)
-{
-	char *thumbnail_path;
-	
-	thumbnail_path = gnome_desktop_thumbnail_path_for_uri (file_uri, GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
-	if (thumbnail_path != NULL) {
-		unlink (thumbnail_path);
-	}
-	g_free (thumbnail_path);
-}
-
 static GdkPixbuf *
 nemo_get_thumbnail_frame (void)
 {
@@ -282,43 +234,6 @@ nemo_thumbnail_frame_image (GdkPixbuf **pixbuf)
 	*pixbuf = pixbuf_with_frame;
 }
 
-GdkPixbuf *
-nemo_thumbnail_unframe_image (GdkPixbuf *pixbuf)
-{
-	GdkPixbuf *pixbuf_without_frame, *frame;
-	int left_offset, top_offset, right_offset, bottom_offset;
-	int w, h;
-		
-	/* The pixbuf isn't already framed (i.e., it was not made by
-	 * an old Nemo), so we must embed it in a frame.
-	 */
-
-	frame = nemo_get_thumbnail_frame ();
-	if (frame == NULL) {
-		return NULL;
-	}
-	
-	left_offset = NEMO_THUMBNAIL_FRAME_LEFT;
-	top_offset = NEMO_THUMBNAIL_FRAME_TOP;
-	right_offset = NEMO_THUMBNAIL_FRAME_RIGHT;
-	bottom_offset = NEMO_THUMBNAIL_FRAME_BOTTOM;
-
-	w = gdk_pixbuf_get_width (pixbuf) - left_offset - right_offset;
-	h = gdk_pixbuf_get_height (pixbuf) - top_offset - bottom_offset;
-	pixbuf_without_frame =
-		gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
-				gdk_pixbuf_get_has_alpha (pixbuf),
-				gdk_pixbuf_get_bits_per_sample (pixbuf),
-				w, h);
-
-	gdk_pixbuf_copy_area (pixbuf,
-			      left_offset, top_offset,
-			      w, h,
-			      pixbuf_without_frame, 0, 0);
-	
-	return pixbuf_without_frame;
-}
-
 void
 nemo_thumbnail_remove_from_queue (const char *file_uri)
 {
@@ -353,45 +268,6 @@ nemo_thumbnail_remove_from_queue (const char *file_uri)
 	pthread_mutex_unlock (&thumbnails_mutex);
 }
 
-void
-nemo_thumbnail_remove_all_from_queue (void)
-{
-	NemoThumbnailInfo *info;
-	GList *l, *next;
-	
-#ifdef DEBUG_THUMBNAILS
-	g_message ("(Remove all from queue) Locking mutex\n");
-#endif
-	pthread_mutex_lock (&thumbnails_mutex);
-
-	/*********************************
-	 * MUTEX LOCKED
-	 *********************************/
-
-	l = thumbnails_to_make.head;
-	while (l != NULL) {
-		info = l->data;
-		next = l->next;
-		if (info != currently_thumbnailing) {
-			g_hash_table_remove (thumbnails_to_make_hash, 
-					     info->image_uri);
-			free_thumbnail_info (info);
-			g_queue_delete_link ((GQueue *)&thumbnails_to_make, l);
-		}
-
-		l = next;
-	}
-	
-	/*********************************
-	 * MUTEX UNLOCKED
-	 *********************************/
-	
-#ifdef DEBUG_THUMBNAILS
-	g_message ("(Remove all from queue) Unlocking mutex\n");
-#endif
-	pthread_mutex_unlock (&thumbnails_mutex);
-}
-
 void
 nemo_thumbnail_prioritize (const char *file_uri)
 {
@@ -440,8 +316,6 @@ thumbnail_thread_notify_file_changed (gpointer image_uri)
 {
 	NemoFile *file;
 
-	gdk_threads_enter ();
-
 	file = nemo_file_get_by_uri ((char *) image_uri);
 #ifdef DEBUG_THUMBNAILS
 	g_message ("(Thumbnail Thread) Notifying file changed file:%p uri: %s\n", file, (char*) image_uri);
@@ -452,13 +326,10 @@ thumbnail_thread_notify_file_changed (gpointer image_uri)
 		nemo_file_invalidate_attributes (file,
 						     NEMO_FILE_ATTRIBUTE_THUMBNAIL |
 						     NEMO_FILE_ATTRIBUTE_INFO);
-        nemo_file_increment_thumbnail_try_count (file);
 		nemo_file_unref (file);
 	}
 	g_free (image_uri);
 
-	gdk_threads_leave ();
-
 	return FALSE;
 }
 
@@ -550,7 +421,7 @@ nemo_can_thumbnail (NemoFile *file)
 }
 
 void
-nemo_create_thumbnail (NemoFile *file)
+nemo_create_thumbnail (NemoFile *file, gint throttle_count)
 {
 	time_t file_mtime = 0;
 	NemoThumbnailInfo *info;
@@ -562,6 +433,7 @@ nemo_create_thumbnail (NemoFile *file)
 	info = g_new0 (NemoThumbnailInfo, 1);
 	info->image_uri = nemo_file_get_uri (file);
 	info->mime_type = nemo_file_get_mime_type (file);
+    info->throttle_count = MIN (10, throttle_count);
 	
 	/* Hopefully the NemoFile will already have the image file mtime,
 	   so we can just use that. Otherwise we have to get it ourselves. */
@@ -703,14 +575,15 @@ thumbnail_thread_start (gpointer data)
 
 		/* Don't try to create a thumbnail if the file was modified recently.
 		   This prevents constant re-thumbnailing of changing files. */ 
-		if (current_time < current_orig_mtime + THUMBNAIL_CREATION_DELAY_SECS &&
+		if (current_time < current_orig_mtime + (THUMBNAIL_CREATION_DELAY_SECS * info->throttle_count) &&
 		    current_time >= current_orig_mtime) {
 #ifdef DEBUG_THUMBNAILS
-			g_message ("(Thumbnail Thread) Skipping: %s\n",
-				   info->image_uri);
+			g_message ("(Thumbnail Thread) Skipping for %d seconds: %s\n",
+                       THUMBNAIL_CREATION_DELAY_SECS * info->throttle_count,
+                       info->image_uri);
 #endif
 			/* Reschedule thumbnailing via a change notification */
-			g_timeout_add_seconds (1, thumbnail_thread_notify_file_changed,
+			g_timeout_add_seconds (THUMBNAIL_CREATION_DELAY_SECS * info->throttle_count, thumbnail_thread_notify_file_changed,
 				       g_strdup (info->image_uri));
  			continue;
 		}
@@ -736,6 +609,7 @@ thumbnail_thread_start (gpointer data)
 										 info->image_uri,
 										 current_orig_mtime);
 		}
+
 		/* We need to call nemo_file_changed(), but I don't think that is
 		   thread safe. So add an idle handler and do it from the main loop. */
 		g_idle_add_full (G_PRIORITY_HIGH_IDLE,
@@ -743,3 +617,9 @@ thumbnail_thread_start (gpointer data)
 				 g_strdup (info->image_uri), NULL);
 	}
 }
+
+gboolean
+nemo_thumbnail_factory_check_status (void)
+{
+    return gnome_desktop_thumbnail_cache_check_permissions (get_thumbnail_factory (), TRUE);
+}
