@@ -40,10 +40,12 @@
 #include "nemo-window-bookmarks.h"
 #include "nemo-window-private.h"
 #include "nemo-desktop-window.h"
-#include "nemo-search-bar.h"
 #include "nemo-location-bar.h"
 #include "nemo-icon-view.h"
 #include "nemo-list-view.h"
+#include "nemo-toolbar.h"
+#include "nemo-plugin-manager.h"
+
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
@@ -70,7 +72,6 @@
 
 #define NETWORK_URI          "network:"
 #define COMPUTER_URI         "computer:"
-#define BURN_CD_URI          "burn:"
 
 static void
 action_close_window_slot_callback (GtkAction *action,
@@ -82,7 +83,7 @@ action_close_window_slot_callback (GtkAction *action,
 	window = NEMO_WINDOW (user_data);
 	slot = nemo_window_get_active_slot (window);
 
-	nemo_window_pane_slot_close (slot->pane, slot);
+	nemo_window_pane_close_slot (slot->pane, slot);
 }
 
 static void
@@ -132,8 +133,8 @@ action_home_callback (GtkAction *action,
 	window = NEMO_WINDOW (user_data);
 	slot = nemo_window_get_active_slot (window);
 
-	nemo_window_slot_go_home (slot, 
-				      nemo_event_should_open_in_new_tab ());
+	nemo_window_slot_go_home (slot,
+				      nemo_event_get_window_open_flags ());
 }
 
 static void
@@ -148,9 +149,8 @@ action_go_to_computer_callback (GtkAction *action,
 	slot = nemo_window_get_active_slot (window);
 
 	computer = g_file_new_for_uri (COMPUTER_URI);
-	nemo_window_slot_go_to (slot,
-				    computer,
-				    nemo_event_should_open_in_new_tab ());
+	nemo_window_slot_open_location (slot, computer,
+					    nemo_event_get_window_open_flags ());
 	g_object_unref (computer);
 }
 
@@ -166,9 +166,8 @@ action_go_to_network_callback (GtkAction *action,
 	slot = nemo_window_get_active_slot (window);
 
 	network = g_file_new_for_uri (NETWORK_URI);
-	nemo_window_slot_go_to (slot,
-				    network,
-				    nemo_event_should_open_in_new_tab ());
+	nemo_window_slot_open_location (slot, network,
+					    nemo_event_get_window_open_flags ());
 	g_object_unref (network);
 }
 
@@ -187,9 +186,8 @@ action_go_to_templates_callback (GtkAction *action,
 	path = nemo_get_templates_directory ();
 	location = g_file_new_for_path (path);
 	g_free (path);
-	nemo_window_slot_go_to (slot,
-				    location,
-				    nemo_event_should_open_in_new_tab ());
+	nemo_window_slot_open_location (slot, location,
+					    nemo_event_get_window_open_flags ());
 	g_object_unref (location);
 }
 
@@ -205,9 +203,8 @@ action_go_to_trash_callback (GtkAction *action,
 	slot = nemo_window_get_active_slot (window);
 
 	trash = g_file_new_for_uri ("trash:///");
-	nemo_window_slot_go_to (slot,
-				    trash,
-				    nemo_event_should_open_in_new_tab ());
+	nemo_window_slot_open_location (slot, trash,
+					    nemo_event_get_window_open_flags ());
 	g_object_unref (trash);
 }
 
@@ -283,6 +280,13 @@ action_preferences_callback (GtkAction *action,
 	nemo_file_management_properties_dialog_show (window);
 }
 
+static void
+action_plugins_callback (GtkAction *action, 
+                         gpointer user_data)
+{
+    nemo_plugin_manager_show ();
+}
+
 static void
 action_about_nemo_callback (GtkAction *action,
 				gpointer user_data)
@@ -300,7 +304,7 @@ action_about_nemo_callback (GtkAction *action,
 		   "along with Nemo; if not, write to the Free Software Foundation, Inc., "
 		   "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA")
 	};
-	gchar *license_trans, *copyright_str;
+	gchar *license_trans;
 	GDateTime *date;
 
 	license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]),
@@ -320,7 +324,6 @@ action_about_nemo_callback (GtkAction *action,
 			      NULL);
 
 	g_free (license_trans);
-	g_free (copyright_str);
 	g_date_time_unref (date);
 }
 
@@ -332,7 +335,7 @@ action_up_callback (GtkAction *action,
 	NemoWindowSlot *slot;
 
 	slot = nemo_window_get_active_slot (window);
-	nemo_window_slot_go_up (slot, FALSE, nemo_event_should_open_in_new_tab ());
+	nemo_window_slot_go_up (slot, nemo_event_get_window_open_flags ());
 }
 
 static void
@@ -477,7 +480,7 @@ action_back_callback (GtkAction *action,
 		      gpointer user_data) 
 {
 	nemo_window_back_or_forward (NEMO_WINDOW (user_data), 
-					 TRUE, 0, nemo_event_should_open_in_new_tab ());
+					 TRUE, 0, nemo_event_get_window_open_flags ());
 }
 
 static void
@@ -485,7 +488,7 @@ action_forward_callback (GtkAction *action,
 			 gpointer user_data) 
 {
 	nemo_window_back_or_forward (NEMO_WINDOW (user_data), 
-					 FALSE, 0, nemo_event_should_open_in_new_tab ());
+					 FALSE, 0, nemo_event_get_window_open_flags ());
 }
 
 static void
@@ -511,7 +514,8 @@ action_split_view_same_location_callback (GtkAction *action,
 	}
 	location = nemo_window_slot_get_location (next_pane->active_slot);
 	if (location) {
-		nemo_window_slot_go_to (nemo_window_get_active_slot (window), location, FALSE);
+		nemo_window_slot_open_location (nemo_window_get_active_slot (window),
+						    location, 0);
 		g_object_unref (location);
 	}
 }
@@ -687,15 +691,17 @@ connect_proxy_cb (GtkActionGroup *action_group,
                   GtkWidget *proxy,
                   NemoWindow *window)
 {
-	GtkLabel *label;
+    GtkWidget *label;
 
 	if (!GTK_IS_MENU_ITEM (proxy))
 		return;
 
-	label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (proxy)));
+    label = gtk_bin_get_child (GTK_BIN (proxy));
 
-	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
-	gtk_label_set_max_width_chars (label, MENU_ITEM_MAX_WIDTH_CHARS);
+    if (GTK_IS_LABEL (label)) {
+       gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+       gtk_label_set_max_width_chars (GTK_LABEL (label), MENU_ITEM_MAX_WIDTH_CHARS);
+    }
 
 	g_signal_connect (proxy, "select",
 			  G_CALLBACK (menu_item_select_cb), window);
@@ -757,7 +763,7 @@ action_new_window_callback (GtkAction *action,
 				application,
 				gtk_window_get_screen (GTK_WINDOW (current_window)));
 
-    nemo_window_slot_open_location (nemo_window_get_active_slot (new_window), loc, 0, NULL);
+    nemo_window_slot_open_location (nemo_window_get_active_slot (new_window), loc, 0);
     g_object_unref (loc);
     g_free (uri);
 }
@@ -784,7 +790,7 @@ toggle_location_entry_setting (NemoWindow     *window,
     GtkActionGroup *action_group;
 
 
-    current_view = nemo_toolbar_get_show_location_entry (pane->tool_bar);
+    current_view = nemo_toolbar_get_show_location_entry (NEMO_TOOLBAR (pane->tool_bar));
     temp_toolbar_visible = pane->temporary_navigation_bar;
     default_toolbar_visible = g_settings_get_boolean (nemo_window_state,
                                                       NEMO_WINDOW_STATE_START_WITH_TOOLBAR);
@@ -793,7 +799,7 @@ toggle_location_entry_setting (NemoWindow     *window,
     grab_focus_only = from_accel_or_menu && (pane->last_focus_widget == NULL || !already_has_focus) && current_view;
 
     if ((temp_toolbar_visible || default_toolbar_visible) && !grab_focus_only) {
-        nemo_toolbar_set_show_location_entry (pane->tool_bar, !current_view);
+        nemo_toolbar_set_show_location_entry (NEMO_TOOLBAR (pane->tool_bar), !current_view);
         g_settings_set_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_LOCATION_ENTRY, !current_view);
 
         action_group = pane->toolbar_action_group;
@@ -818,6 +824,13 @@ action_toggle_location_entry_callback (GtkToggleAction *action,
     toggle_location_entry_setting(window, pane, FALSE);
 }
 
+void nemo_window_show_location_entry (NemoWindow *window) {
+	NemoWindowPane *pane;
+
+    pane = nemo_window_get_active_pane (window);
+    toggle_location_entry_setting(window, pane, TRUE);
+}
+
 static void
 action_menu_edit_location_callback (GtkAction *action,
 				gpointer user_data)
@@ -991,26 +1004,34 @@ action_tabs_next_callback (GtkAction *action,
 	nemo_notebook_set_current_page_relative (NEMO_NOTEBOOK (pane->notebook), 1);
 }
 
+static void
+reorder_tab (NemoWindowPane *pane, int offset)
+{
+	int page_num;
+
+	g_return_if_fail (pane != NULL);
+
+	page_num = gtk_notebook_get_current_page (
+		GTK_NOTEBOOK (pane->notebook));
+	g_return_if_fail (page_num != -1);
+	nemo_notebook_reorder_child_relative (
+		NEMO_NOTEBOOK (pane->notebook), page_num, offset);
+}
+
 static void
 action_tabs_move_left_callback (GtkAction *action,
 				gpointer user_data)
 {
-	NemoWindowPane *pane;
 	NemoWindow *window = user_data;
-
-	pane = nemo_window_get_active_pane (window);
-	nemo_notebook_reorder_current_child_relative (NEMO_NOTEBOOK (pane->notebook), -1);
+	reorder_tab (nemo_window_get_active_pane (window), -1);
 }
 
 static void
 action_tabs_move_right_callback (GtkAction *action,
 				 gpointer user_data)
 {
-	NemoWindowPane *pane;
 	NemoWindow *window = user_data;
-
-	pane = nemo_window_get_active_pane (window);
-	nemo_notebook_reorder_current_child_relative (NEMO_NOTEBOOK (pane->notebook), 1);
+	reorder_tab (nemo_window_get_active_pane (window), 1);
 }
 
 static void
@@ -1031,6 +1052,47 @@ action_tab_change_action_activate_callback (GtkAction *action,
 	}
 }
 
+static void
+action_new_folder_callback (GtkAction *action,
+                            gpointer user_data)
+{
+    g_assert (NEMO_IS_WINDOW (user_data));
+    NemoWindow *window = user_data;
+    NemoView *view = get_current_view (window);
+
+    nemo_view_new_folder (view);
+}
+
+static void
+open_in_terminal_other (const gchar *path)
+{
+    gchar *argv[2];
+    argv[0] = g_settings_get_string (gnome_terminal_preferences, GNOME_DESKTOP_TERMINAL_EXEC);
+    argv[1] = NULL;
+    g_spawn_async(path, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL);
+}
+
+
+static void
+action_open_terminal_callback(GtkAction *action, gpointer callback_data)
+{  
+    NemoWindow *window;
+    NemoView *view;
+    
+    window = NEMO_WINDOW(callback_data);
+
+    view = get_current_view (window);
+
+    gchar *path;
+    gchar *uri = nemo_view_get_uri (view);
+    GFile *gfile = g_file_new_for_uri (uri);
+    path = g_file_get_path (gfile);
+    open_in_terminal_other (path);
+    g_free (uri);
+    g_free (path);
+    g_object_unref (gfile);
+}
+
 static const GtkActionEntry main_entries[] = {
   /* name, stock id, label */  { "File", NULL, N_("_File") },
   /* name, stock id, label */  { "Edit", NULL, N_("_Edit") },
@@ -1044,6 +1106,10 @@ static const GtkActionEntry main_entries[] = {
                                  N_("Prefere_nces"),               
                                  NULL, N_("Edit Nemo preferences"),
                                  G_CALLBACK (action_preferences_callback) },
+                               { NEMO_ACTION_PLUGIN_MANAGER, NULL,
+                                 N_("Plugins"),               
+                                 "<alt>p", N_("Manage extensions, actions and scripts"),
+                                 G_CALLBACK (action_plugins_callback) },
 #ifdef TEXT_CHANGE_UNDO
   /* name, stock id, label */  { "Undo", NULL, N_("_Undo"),
                                  "<control>Z", N_("Undo the last text change"),
@@ -1068,20 +1134,20 @@ static const GtkActionEntry main_entries[] = {
   /* tooltip */                  N_("Display Nemo help"),
                                  G_CALLBACK (action_nemo_manual_callback) },
   /** name, stock id          { "NemoHelpSearch", NULL,
-  /* label, accelerator        N_("Search for files"), NULL,
-  /* tooltip                   N_("Locate files based on file name and type. Save your searches for later use."),
+     label, accelerator        N_("Search for files"), NULL,
+     tooltip                   N_("Locate files based on file name and type. Save your searches for later use."),
                                  G_CALLBACK (action_nemo_manual_callback) },
-  /* name, stock id          { "NemoHelpSort", NULL,
-  /* label, accelerator        N_("Sort files and folders"), NULL,
-  /* tooltip                   N_("Arrange files by name, size, type, or when they were changed."),
+     name, stock id          { "NemoHelpSort", NULL,
+     label, accelerator        N_("Sort files and folders"), NULL,
+     tooltip                   N_("Arrange files by name, size, type, or when they were changed."),
                                  G_CALLBACK (action_nemo_manual_callback) },
-  /* name, stock id          { "NemoHelpLost", NULL,
-  /* label, accelerator        N_("Find a lost file"), NULL,
-  /* tooltip                   N_("Follow these tips if you can't find a file you created or downloaded."),
+     name, stock id          { "NemoHelpLost", NULL,
+     label, accelerator        N_("Find a lost file"), NULL,
+     tooltip                   N_("Follow these tips if you can't find a file you created or downloaded."),
                                  G_CALLBACK (action_nemo_manual_callback) },
-  /* name, stock id          { "NemoHelpShare", NULL,
-  /* label, accelerator        N_("Share and transfer files"), NULL,
-  /* tooltip                   N_("Easily transfer files to your contacts and devices from the file manager."),
+     name, stock id          { "NemoHelpShare", NULL,
+     label, accelerator        N_("Share and transfer files"), NULL,
+     tooltip                   N_("Easily transfer files to your contacts and devices from the file manager."),
                                  G_CALLBACK (action_nemo_manual_callback) }, **/
   /* name, stock id */         { "About Nemo", GTK_STOCK_ABOUT,
   /* label, accelerator */       N_("_About"), NULL,
@@ -1153,8 +1219,8 @@ static const GtkActionEntry main_entries[] = {
   /* name, stock id, label */  { NEMO_ACTION_FORWARD, GTK_STOCK_GO_FORWARD, N_("_Forward"),
 				 "<alt>Right", N_("Go to the next visited location"),
 				 G_CALLBACK (action_forward_callback) },
-  /* name, stock id, label */  { NEMO_ACTION_EDIT_LOCATION, NULL, N_("_Location..."),
-                                 "<control>L", N_("Specify a location to open"),
+  /* name, stock id, label */  { NEMO_ACTION_EDIT_LOCATION, NULL, N_("Toggle _Location Entry"),
+                                 "<control>L", N_("Switch between location entry and breadcrumbs"),
                                  G_CALLBACK (action_menu_edit_location_callback) },
   /* name, stock id, label */  { "SplitViewNextPane", NULL, N_("S_witch to Other Pane"),
 				 "F6", N_("Move focus to the other pane in a split view window"),
@@ -1233,7 +1299,6 @@ static const GtkRadioActionEntry main_radio_entries[] = {
 GtkActionGroup *
 nemo_window_create_toolbar_action_group (NemoWindow *window)
 {
-	gboolean show_label_search_icon_toolbar;
     gboolean show_location_entry_initially;
 
 	NemoNavigationState *navigation_state;
@@ -1246,7 +1311,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
 	action = g_object_new (NEMO_TYPE_NAVIGATION_ACTION,
 			       "name", NEMO_ACTION_BACK,
 			       "label", _("_Back"),
-			       "stock_id", GTK_STOCK_GO_BACK,
+			       "icon_name", "go-previous-symbolic",
 			       "tooltip", _("Go to the previous visited location"),
 			       "arrow-tooltip", _("Back history"),
 			       "window", window,
@@ -1262,7 +1327,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
 	action = g_object_new (NEMO_TYPE_NAVIGATION_ACTION,
 			       "name", NEMO_ACTION_FORWARD,
 			       "label", _("_Forward"),
-			       "stock_id", GTK_STOCK_GO_FORWARD,
+			       "icon_name", "go-next-symbolic",
 			       "tooltip", _("Go to the next visited location"),
 			       "arrow-tooltip", _("Forward history"),
 			       "window", window,
@@ -1281,7 +1346,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
    	action = g_object_new (NEMO_TYPE_NAVIGATION_ACTION,
    			       "name", NEMO_ACTION_UP,
    			       "label", _("_Up"),
-   			       "stock_id", GTK_STOCK_GO_UP,
+   			       "icon_name", "go-up-symbolic",
    			       "tooltip", _("Go to parent folder"),
    			       "arrow-tooltip", _("Forward history"),
    			       "window", window,
@@ -1296,7 +1361,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
    	action = g_object_new (NEMO_TYPE_NAVIGATION_ACTION,
    			       "name", NEMO_ACTION_RELOAD,
    			       "label", _("_Reload"),
-   			       "stock_id", GTK_STOCK_REFRESH,
+   			       "icon_name", "view-refresh-symbolic",
    			       "tooltip", _("Reload the current location"),
    			       "window", window,
    			       "direction", NEMO_NAVIGATION_DIRECTION_RELOAD,
@@ -1310,7 +1375,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
    	action = g_object_new (NEMO_TYPE_NAVIGATION_ACTION,
    			       "name", NEMO_ACTION_HOME,
    			       "label", _("_Home"),
-   			       "stock_id", GTK_STOCK_HOME,
+   			       "icon_name", "go-home-symbolic",
    			       "tooltip", _("Go to home directory"),
    			       "window", window,
    			       "direction", NEMO_NAVIGATION_DIRECTION_HOME,
@@ -1324,7 +1389,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
    	action = g_object_new (NEMO_TYPE_NAVIGATION_ACTION,
    			       "name", NEMO_ACTION_COMPUTER,
    			       "label", _("_Computer"),
-   			       "icon_name", "computer",
+   			       "icon_name", "computer-symbolic",
    			       "tooltip", _("Go to Computer"),
    			       "window", window,
    			       "direction", NEMO_NAVIGATION_DIRECTION_COMPUTER,
@@ -1337,7 +1402,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
 
     action = GTK_ACTION (gtk_toggle_action_new (NEMO_ACTION_TOGGLE_LOCATION,
                                                 _("Location"),
-                                                _("Use Location Entry"),
+                                                _("Toggle Location Entry"),
                                                 NULL));
     gtk_action_group_add_action (action_group, GTK_ACTION (action));
     show_location_entry_initially = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_LOCATION_ENTRY);
@@ -1348,6 +1413,27 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
 
     g_object_unref (action);
 
+    action = GTK_ACTION (gtk_action_new (NEMO_ACTION_NEW_FOLDER,
+                                                _("New folder"),
+                                                _("Create a new folder"),
+                                                NULL));
+    gtk_action_group_add_action (action_group, GTK_ACTION (action));
+    g_signal_connect (action, "activate",
+                      G_CALLBACK (action_new_folder_callback), window);
+    gtk_action_set_icon_name (GTK_ACTION (action), "folder-symbolic");
+    g_object_unref (action);
+    
+    action = GTK_ACTION (gtk_action_new (NEMO_ACTION_OPEN_IN_TERMINAL,
+                                                _("Open in Terminal"),
+                                                _("Open a terminal in the active folder"),
+                                                NULL));
+    gtk_action_group_add_action (action_group, GTK_ACTION (action));
+    g_signal_connect (action, "activate",
+                      G_CALLBACK (action_open_terminal_callback), window);
+    gtk_action_set_icon_name (GTK_ACTION (action), "utilities-terminal-symbolic");
+    g_object_unref (action);
+
+
     action = GTK_ACTION (gtk_toggle_action_new (NEMO_ACTION_ICON_VIEW,
                          _("Icons"),
                          _("Icon View"),
@@ -1388,11 +1474,7 @@ nemo_window_create_toolbar_action_group (NemoWindow *window)
  				NULL));
  
   	gtk_action_group_add_action (action_group, action);
-    gtk_action_set_icon_name (GTK_ACTION (action), "edit-find");
- 
- 
- 	show_label_search_icon_toolbar = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_LABEL_SEARCH_ICON_TOOLBAR);
- 	gtk_action_set_is_important (GTK_ACTION (action), show_label_search_icon_toolbar);
+    gtk_action_set_icon_name (GTK_ACTION (action), "edit-find-symbolic");
   
   	g_object_unref (action);
 
@@ -1564,11 +1646,7 @@ nemo_window_initialize_menus (NemoWindow *window)
 void
 nemo_window_finalize_menus (NemoWindow *window)
 {
-	NemoTrashMonitor *monitor;
-
-	monitor = nemo_trash_monitor_get ();
-
-	g_signal_handlers_disconnect_by_func (monitor,
+	g_signal_handlers_disconnect_by_func (nemo_trash_monitor_get(),
 					      trash_state_changed_cb, window);
 }
 
@@ -1623,7 +1701,7 @@ add_extension_menu_items (NemoWindow *window,
 		
 		g_object_get (item, "menu", &menu, NULL);
 		
-		action = nemo_action_from_menu_item (item);
+		action = nemo_action_from_menu_item (item, GTK_WIDGET (window));
 		gtk_action_group_add_action_with_accel (action_group, action, NULL);
 		
 		path = g_build_path ("/", POPUP_PATH_EXTENSION_ACTIONS, subdirectory, NULL);
